@@ -10,6 +10,7 @@ from .script import BaseScript
 
 
 logger = logging.getLogger(__name__)
+logger.setLevel(logging.INFO)
 
 
 class WorkFlowManager(BaseScript, abc.ABC):
@@ -37,9 +38,13 @@ class WorkFlowManager(BaseScript, abc.ABC):
         super().__init__()
 
     def set_flow_id(self, args, default=None):
-        default = default or str(uuid4())
+        default = default or self._generate_flow_id()
         super().set_flow_id(args, default)
 
+    @staticmethod
+    def _generate_flow_id():
+        return str(uuid4())
+
     @property
     def max_running_jobs(self):
         return self.args.max_running_jobs
@@ -79,11 +84,6 @@ class WorkFlowManager(BaseScript, abc.ABC):
                 else:
                     still_running[key] = False
 
-    def collect_running_jobs(self):
-        """
-        Get all running jobs with same flow id
-        """
-
     def on_start(self):
         pass
 
