@@ -4,7 +4,7 @@ from typing import List, Generator
 from operator import itemgetter
 from glob import iglob
 from os import listdir, remove, environ, makedirs
-from os.path import exists as os_exists, dirname, getctime
+from os.path import exists as os_exists, dirname, getctime, basename
 from shutil import copyfile
 from datetime import datetime, timedelta, timezone
 
@@ -22,6 +22,7 @@ DEFAULT_REGION = "us-west-2"
 
 
 logger = logging.getLogger(__name__)
+logger.setLevel(logging.INFO)
 
 
 def just_log_exception(exception):
@@ -66,15 +67,18 @@ def s3_credentials(key, secret, include_region=False):
 
 
 def get_file(path, *args, aws_key=None, aws_secret=None, **kwargs):
+    op_kwargs = kwargs.pop("op_kwargs", {})
     if path.startswith(_S3_ATTRIBUTE):
         fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
-        fp = fs.open(s3_path(path), *args)
+        if "ACL" in op_kwargs:
+            op_kwargs["acl"] = op_kwargs.pop("ACL")
+        fp = fs.open(s3_path(path), *args, **op_kwargs)
         try:
             fp.name = path
         except Exception:
             pass
     else:
-        fp = open(path, *args)
+        fp = open(path, *args, **op_kwargs)
 
     return fp
 
@@ -83,8 +87,10 @@ DOWNLOAD_CHUNK_SIZE = 500 * 1024 * 1024
 UPLOAD_CHUNK_SIZE = 100 * 1024 * 1024
 
 
-def download_file(path, dest, aws_key=None, aws_secret=None, **kwargs):
+def download_file(path, dest=None, aws_key=None, aws_secret=None, **kwargs):
     assert path.startswith(_S3_ATTRIBUTE), f"Not a s3 source: {path}"
+    if dest is None:
+        dest = basename(path)
     with get_file(path, "rb", aws_key=aws_key, aws_secret=aws_secret, **kwargs) as r, open(dest, "wb") as w:
         total = 0
         while True:
@@ -97,6 +103,8 @@ def download_file(path, dest, aws_key=None, aws_secret=None, **kwargs):
 
 def upload_file(path, dest, aws_key=None, aws_secret=None, **kwargs):
     assert dest.startswith(_S3_ATTRIBUTE), f"Not a s3 source: {dest}"
+    if dest.endswith("/"):
+        dest = dest + basename(path)
     with open(path, "rb") as r, get_file(dest, "wb", aws_key=aws_key, aws_secret=aws_secret, **kwargs) as w:
         total = 0
         while True:
@@ -119,16 +127,18 @@ def get_glob(path, aws_key=None, aws_secret=None, **kwargs):
 
 def cp_file(src_path, dest_path, aws_key=None, aws_secret=None, **kwargs):
     if src_path.startswith(_S3_ATTRIBUTE):
+        op_kwargs = kwargs.pop("op_kwargs", {})
         fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
-        fs.copy(s3_path(src_path), s3_path(dest_path))
+        fs.copy(s3_path(src_path), s3_path(dest_path), **op_kwargs)
     else:
         copyfile(src_path, dest_path)
 
 
 def mv_file(src_path, dest_path, aws_key=None, aws_secret=None, **kwargs):
     if src_path.startswith(_S3_ATTRIBUTE) and dest_path.startswith(_S3_ATTRIBUTE):
-        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret))
-        fs.mv(s3_path(src_path), s3_path(dest_path))
+        op_kwargs = kwargs.pop("op_kwargs", {})
+        fs = S3FileSystem(**s3_credentials(aws_key, aws_secret), **kwargs)
+        fs.mv(s3_path(src_path), s3_path(dest_path), **op_kwargs)
     elif src_path.startswith(_S3_ATTRIBUTE):
         download_file(src_path, dest_path, aws_key, aws_secret, **kwargs)
         rm_file(src_path)
@@ -177,7 +187,7 @@ def list_folder_in_ts_order(input_folder: str, aws_key=None, aws_secret=None, **
     results = []
     for input_file in list_folder(input_folder, aws_key=aws_key, aws_secret=aws_secret, **kwargs):
         if input_folder.startswith(_S3_ATTRIBUTE):
-            with get_file(input_file, "rb") as f:
+            with get_file(input_file, "rb", aws_key=aws_key, aws_secret=aws_secret, **kwargs) as f:
                 results.append((input_file, f.info()["LastModified"]))
         else:
             results.append((input_file, getctime(input_file)))
@@ -243,11 +253,11 @@ class S3SessionFactory:
     Generate s3 temporal session credentials in cases where that is required
     """
 
-    def __init__(self, aws_key, aws_secret, aws_role, external_id, expiration_margin_minutes=10):
+    def __init__(self, aws_key, aws_secret, aws_role, aws_external_id, expiration_margin_minutes=10):
         self.aws_role = aws_role
         self.aws_key = aws_key
         self.aws_secret = aws_secret
-        self.external_id = external_id
+        self.aws_external_id = aws_external_id
         self.expiration_margin_minutes = expiration_margin_minutes
         self.credentials_expiration = None
         self.sts_client = boto3.client("sts", aws_access_key_id=aws_key, aws_secret_access_key=aws_secret)
@@ -263,7 +273,7 @@ class S3SessionFactory:
         ):
             session_name = str(uuid.uuid4())
             self.full_credentials = self.sts_client.assume_role(
-                RoleArn=self.aws_role, RoleSessionName=session_name, ExternalId=self.external_id,
+                RoleArn=self.aws_role, RoleSessionName=session_name, ExternalId=self.aws_external_id,
             )["Credentials"]
             self.credentials_expiration = self.full_credentials["Expiration"]
         return {
@@ -271,3 +281,40 @@ class S3SessionFactory:
             "aws_secret": self.full_credentials["SecretAccessKey"],
             "token": self.full_credentials["SessionToken"],
         }
+
+
+class S3Helper:
+    def __init__(self, aws_key, aws_secret, aws_role=None, aws_external_id=None, expiration_margin_minutes=10):
+        self.s3_session_factory = None
+        self.credentials = {"aws_key": aws_key, "aws_secret": aws_secret}
+        if aws_role is not None and aws_external_id is not None:
+            self.s3_session_factory = S3SessionFactory(
+                aws_key, aws_secret, aws_role, aws_external_id, expiration_margin_minutes
+            )
+
+        for method in (
+            get_file,
+            upload_file,
+            download_file,
+            list_folder,
+            mv_file,
+            rm_file,
+            get_glob,
+            empty_folder,
+            exists,
+            touch,
+            cp_file,
+            list_folder_in_ts_order,
+            list_folder_files_recursive,
+            s3_folder_size,
+        ):
+            self._wrap_method(method)
+
+    def _wrap_method(self, method):
+        def _method(*args, **kwargs):
+            if self.s3_session_factory is not None:
+                self.credentials = self.s3_session_factory.get_credentials()
+            kwargs.update(self.credentials)
+            return method(*args, **kwargs)
+
+        setattr(self, method.__name__, _method)
