@@ -12,7 +12,7 @@ from typing import List
 
 from scrapinghub import ScrapinghubClient, DuplicateJobError
 
-from .utils import (
+from shub_workflow.utils import (
     resolve_project_id,
     dash_retry_decorator,
 )
@@ -20,13 +20,16 @@ from .utils import (
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
-logging.basicConfig(format="%(asctime)s [%(levelname)s] %(message)s")
 
 
 class ArgumentParserScript(abc.ABC):
     def __init__(self):
         self.args = self.parse_args()
 
+    @property
+    def description(self):
+        return "You didn't set description for this script. Please set description property accordingly."
+
     @abc.abstractmethod
     def add_argparser_options(self):
         pass
@@ -41,29 +44,32 @@ class ArgumentParserScript(abc.ABC):
     def run(self):
         pass
 
+
 class BaseScript(ArgumentParserScript):
 
-    name = ''  # optional, may be needed for some applications
+    name = ""  # optional, may be needed for some applications
     flow_id_required = False  # if True, script can only run in the context of a flow_id
     children_tags = None
-    default_project_id = None
+    default_project_id = None  # If None, autodetect (see shub_workflow.utils.resolve_project_id)
 
     def __init__(self):
         self.project_id = None
         self.client = ScrapinghubClient()
+        self.close_reason = None
+        self.__flow_tags = []
         super().__init__()
 
+    def append_flow_tag(self, tag):
+        self.__flow_tags.append(tag)
+        self.add_job_tags(tags=[tag])
+
     def set_flow_id(self, args, default=None):
-        self._flow_id = args.flow_id or self.get_flowid_from_tags() or default
+        self._flow_id = args.flow_id or self._get_flowid_from_tags() or default
         if self.flow_id_required:
             assert not self.flow_id_required or self.flow_id, "Could not detect flow_id. Please set with --flow-id."
         if self.flow_id:
             self.add_job_tags(tags=[f"FLOW_ID={self.flow_id}"])
 
-    @property
-    def description(self):
-        return "You didn't set description for this script. Please set description property accordingly."
-
     @property
     def flow_id(self):
         return self._flow_id
@@ -100,11 +106,12 @@ class BaseScript(ArgumentParserScript):
         return self.client.get_project(project_id or self.project_id)
 
     @dash_retry_decorator
-    def get_job_metadata(self, jobid=None, project_id=None):
+    def get_job_metadata(self, jobid=None):
         """If jobid is None, get own metadata
         """
         jobid = jobid or os.getenv("SHUB_JOBKEY")
         if jobid:
+            project_id = jobid.split("/", 1)[0]
             project = self.get_project(project_id)
             job = project.jobs.get(jobid)
             return job.metadata
@@ -121,25 +128,25 @@ class BaseScript(ArgumentParserScript):
             return project.jobs.get(jobid)
         logger.warning("SHUB_JOBKEY not set: not running on ScrapyCloud.")
 
-    def get_job_tags(self, jobid=None, project_id=None):
+    def get_job_tags(self, jobid=None):
         """If jobid is None, get own tags
         """
-        metadata = self.get_job_metadata(jobid, project_id)
+        metadata = self.get_job_metadata(jobid)
         if metadata:
             return dict(metadata.list()).get("tags", [])
         return []
 
     @staticmethod
     @dash_retry_decorator
-    def update_metadata(metadata, data):
+    def _update_metadata(metadata, data):
         metadata.update(data)
 
-    def add_job_tags(self, jobid=None, project_id=None, tags=None):
+    def add_job_tags(self, jobid=None, tags=None):
         """If jobid is None, add tags to own list of tags.
         """
         if tags:
             update = False
-            job_tags = self.get_job_tags(jobid, project_id)
+            job_tags = self.get_job_tags(jobid)
             for tag in tags:
                 if tag not in job_tags:
                     if tag.startswith("FLOW_ID="):
@@ -148,14 +155,14 @@ class BaseScript(ArgumentParserScript):
                         job_tags.append(tag)
                     update = True
             if update:
-                metadata = self.get_job_metadata(jobid, project_id)
+                metadata = self.get_job_metadata(jobid)
                 if metadata:
-                    self.update_metadata(metadata, {"tags": job_tags})
+                    self._update_metadata(metadata, {"tags": job_tags})
 
-    def get_flowid_from_tags(self, jobid=None, project_id=None):
+    def _get_flowid_from_tags(self, jobid=None):
         """If jobid is None, get flowid from own tags
         """
-        for tag in self.get_job_tags(jobid, project_id):
+        for tag in self.get_job_tags(jobid):
             if tag.startswith("FLOW_ID="):
                 return tag.replace("FLOW_ID=", "")
 
@@ -164,7 +171,8 @@ class BaseScript(ArgumentParserScript):
         tags.extend(self.args.tag)
         if self.flow_id:
             tags.append(f"FLOW_ID={self.flow_id}")
-        return list(set(tags)) or None
+        tags.extend(self.__flow_tags)
+        return sorted(set(tags)) or None
 
     @dash_retry_decorator
     def _schedule_job(self, spider: str, tags=None, units=None, project_id=None, **kwargs):
@@ -210,10 +218,11 @@ class BaseScript(ArgumentParserScript):
         return self.get_jobs(project_id, **kwargs)
 
     @dash_retry_decorator
-    def is_running(self, jobkey, project_id=None):
+    def is_running(self, jobkey):
         """
         Checks whether a job is running (or pending)
         """
+        project_id = jobkey.split("/", 1)[0]
         project = self.get_project(project_id)
         job = project.jobs.get(jobkey)
         if job.metadata.get("state") in ("running", "pending"):
@@ -221,10 +230,11 @@ class BaseScript(ArgumentParserScript):
         return False
 
     @dash_retry_decorator
-    def is_finished(self, jobkey, project_id=None):
+    def is_finished(self, jobkey):
         """
         Checks whether a job is running. if so, return close_reason. Otherwise return None.
         """
+        project_id = jobkey.split("/", 1)[0]
         project = self.get_project(project_id)
         job = project.jobs.get(jobkey)
         if job.metadata.get("state") == "finished":
@@ -233,6 +243,10 @@ class BaseScript(ArgumentParserScript):
     @dash_retry_decorator
     def finish(self, jobid=None, close_reason=None):
         close_reason = close_reason or "finished"
+        if jobid is None:
+            self.close_reason = close_reason
+            if close_reason == "finished":
+                return
         jobid = jobid or os.getenv("SHUB_JOBKEY")
         if jobid:
             project_id = jobid.split("/", 1)[0]
