@@ -1,6 +1,7 @@
 import os
 import re
 from io import StringIO
+from collections import namedtuple
 
 from unittest import TestCase
 from unittest.mock import patch, Mock, call
@@ -10,6 +11,8 @@ from shub_workflow.graph.task import Task, SpiderTask, Resource
 
 from .utils.contexts import script_args
 
+Job = namedtuple("Job", "key")
+
 
 class TestManager(GraphManager):
 
@@ -102,10 +105,9 @@ class TestManager3(GraphManager):
         # return root jobs
         return (jobA,)
 
-    def get_job_tags(self, jobid=None, project_id=None):
-        if jobid is None:
-            return ["FLOW_ID=mytagsflowid"]
-        return super().get_job_tags(jobid, project_id)
+    @staticmethod
+    def _generate_flow_id():
+        return "mygeneratedflowid"
 
 
 class BaseTestCase(TestCase):
@@ -211,7 +213,7 @@ class ManagerTest(BaseTestCase):
         """
         with script_args(["-s", "jobA", "--root-jobs"]):
             with self.assertRaises(SystemExit):
-                manager = TestManager()
+                TestManager()
         self.assertTrue("You can't provide both --starting-job and --root-jobs" in mock_stderr.getvalue())
 
     def test_root_jobs(self):
@@ -454,78 +456,124 @@ class ManagerTest(BaseTestCase):
     def test_tags(self):
         with script_args(["--starting-job=jobA", "--tag=tag3", "--tag=tag4"]):
             manager = TestManager3()
+        self.assertEqual(manager.flow_id, "mygeneratedflowid")
+        project = Mock()
+        manager.get_project = lambda _: project
         manager.is_finished = lambda x: None
-        manager.schedule_script = Mock()
-        manager.schedule_script.side_effect = [
-            "999/1/1",
-            "999/1/2",
-            "999/1/3",
-            "999/1/4",
+        project.jobs.run.side_effect = [
+            Job("999/1/1"),
+            Job("999/1/2"),
+            Job("999/1/3"),
+            Job("999/1/4"),
         ]
         manager._WorkFlowManager__on_start()
 
         # first loop
         result = manager.workflow_loop()
         self.assertTrue(result)
-        self.assertEqual(manager.schedule_script.call_count, 4)
+        self.assertEqual(project.jobs.run.call_count, 4)
         for i in range(4):
-            manager.schedule_script.assert_any_call(
-                ["commandA", f"--parg={i}", "argA", "--optionA"], tags=["tag1", "tag2"], units=None, project_id=None,
+            project.jobs.run.assert_any_call(
+                spider="py:commandA",
+                add_tag=["FLOW_ID=mygeneratedflowid", "tag1", "tag2", "tag3", "tag4"],
+                units=None,
+                cmd_args=f"--parg={i} argA --optionA",
+                meta=None,
             )
-        self.assertEqual(
-            set(manager._make_tags(["tag1", "tag2"])),
-            set(["tag1", "tag2", "tag3", "tag4", f"FLOW_ID={manager.flow_id}"]),
-        )
 
     def test_flow_id_from_command_line(self):
-        with script_args(["--starting-job=jobA", "--flow-id=myflowid"]):
+        with script_args(["--starting-job=jobA", "--flow-id=myclflowid"]):
             manager = TestManager3()
+        self.assertEqual(manager.flow_id, "myclflowid")
+        project = Mock()
+        manager.get_project = lambda _: project
         manager.is_finished = lambda x: None
-        manager.schedule_script = Mock()
-        manager.schedule_script.side_effect = [
-            "999/1/1",
-            "999/1/2",
-            "999/1/3",
-            "999/1/4",
+        project.jobs.run.side_effect = [
+            Job("999/1/1"),
+            Job("999/1/2"),
+            Job("999/1/3"),
+            Job("999/1/4"),
         ]
         manager._WorkFlowManager__on_start()
 
         # first loop
         result = manager.workflow_loop()
         self.assertTrue(result)
-        self.assertEqual(manager.schedule_script.call_count, 4)
+        self.assertEqual(project.jobs.run.call_count, 4)
         for i in range(4):
-            manager.schedule_script.assert_any_call(
-                ["commandA", f"--parg={i}", "argA", "--optionA"], tags=["tag1", "tag2"], units=None, project_id=None,
+            project.jobs.run.assert_any_call(
+                spider="py:commandA",
+                add_tag=["FLOW_ID=myclflowid", "tag1", "tag2"],
+                units=None,
+                cmd_args=f"--parg={i} argA --optionA",
+                meta=None,
             )
-        self.assertEqual(
-            set(manager._make_tags(["tag1", "tag2"])), set(["tag1", "tag2", "FLOW_ID=myflowid"]),
-        )
 
     def test_flow_id_from_job_tags(self):
+        class _TestManager(TestManager3):
+            def get_job_tags(self, jobid=None):
+                if jobid is None:
+                    return ["FLOW_ID=myflowidfromtag"]
+
         with script_args(["--starting-job=jobA"]):
-            manager = TestManager3()
+            manager = _TestManager()
+        self.assertEqual(manager.flow_id, "myflowidfromtag")
+        project = Mock()
+        manager.get_project = lambda _: project
         manager.is_finished = lambda x: None
-        manager.schedule_script = Mock()
-        manager.schedule_script.side_effect = [
-            "999/1/1",
-            "999/1/2",
-            "999/1/3",
-            "999/1/4",
+        project.jobs.run.side_effect = [
+            Job("999/1/1"),
+            Job("999/1/2"),
+            Job("999/1/3"),
+            Job("999/1/4"),
         ]
         manager._WorkFlowManager__on_start()
 
         # first loop
         result = manager.workflow_loop()
         self.assertTrue(result)
-        self.assertEqual(manager.schedule_script.call_count, 4)
+        self.assertEqual(project.jobs.run.call_count, 4)
         for i in range(4):
-            manager.schedule_script.assert_any_call(
-                ["commandA", f"--parg={i}", "argA", "--optionA"], tags=["tag1", "tag2"], units=None, project_id=None,
+            project.jobs.run.assert_any_call(
+                spider="py:commandA",
+                add_tag=["FLOW_ID=myflowidfromtag", "tag1", "tag2"],
+                units=None,
+                cmd_args=f"--parg={i} argA --optionA",
+                meta=None,
             )
-        self.assertEqual(
-            set(manager._make_tags(["tag1", "tag2"])), set(["tag1", "tag2", "FLOW_ID=mytagsflowid"]),
-        )
+
+    def test_additional_workflow_tags(self):
+        class _TestManager(TestManager3):
+            add_job_tags = Mock()
+
+        with script_args(["--starting-job=jobA"]):
+            manager = _TestManager()
+            manager.append_flow_tag("EXEC_ID=myexecid")
+        self.assertEqual(manager.flow_id, "mygeneratedflowid")
+        project = Mock()
+        manager.get_project = lambda _: project
+        manager.is_finished = lambda x: None
+        project.jobs.run.side_effect = [
+            Job("999/1/1"),
+            Job("999/1/2"),
+            Job("999/1/3"),
+            Job("999/1/4"),
+        ]
+        manager._WorkFlowManager__on_start()
+
+        # first loop
+        result = manager.workflow_loop()
+        self.assertTrue(result)
+        self.assertEqual(project.jobs.run.call_count, 4)
+        for i in range(4):
+            project.jobs.run.assert_any_call(
+                spider="py:commandA",
+                add_tag=["EXEC_ID=myexecid", "FLOW_ID=mygeneratedflowid", "tag1", "tag2"],
+                units=None,
+                cmd_args=f"--parg={i} argA --optionA",
+                meta=None,
+            )
+        manager.add_job_tags.assert_any_call(tags=["EXEC_ID=myexecid"])
 
     def test_skip_job(self):
         with script_args(["--starting-job=jobA", "--skip-job=jobC"]):
@@ -1244,7 +1292,7 @@ class ManagerTest(BaseTestCase):
                 return jobA, jobB, jobC, jobD
 
         with script_args(
-            ["--starting-job=jobA", "--starting-job=jobB", "--starting-job=jobC", "--starting-job=jobD",]
+            ["--starting-job=jobA", "--starting-job=jobB", "--starting-job=jobC", "--starting-job=jobD"]
         ):
             manager = _TestManager()
 
@@ -1583,7 +1631,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(result)
         for i in range(4):
             manager.schedule_script.assert_any_call(
-                ["commandB", f'--config={{"topic": {i}, "file": "ds_dump_{i}"}}', "argB", "--optionB",],
+                ["commandB", f'--config={{"topic": {i}, "file": "ds_dump_{i}"}}', "argB", "--optionB"],
                 tags=None,
                 units=None,
                 project_id=None,
@@ -1618,7 +1666,7 @@ class ManagerTest(BaseTestCase):
         self.assertTrue(result)
         for i in range(4):
             manager.schedule_script.assert_any_call(
-                ["commandB", f'--config={{"topic": {i}, "file": "ds_dump_{i}"}}', "argB", "--optionB",],
+                ["commandB", f'--config={{"topic": {i}, "file": "ds_dump_{i}"}}', "argB", "--optionB"],
                 tags=None,
                 units=None,
                 project_id=1999,
@@ -1698,3 +1746,38 @@ class ManagerTest(BaseTestCase):
         manager.schedule_spider.assert_any_call(
             "myspiderS", tags=["tag1"], units=1, project_id=None, argA="valA", argB="valB",
         )
+
+    def test_add_task_on_start(self):
+        class _TestManager(GraphManager):
+            project_id = 999
+            name = "test"
+
+            def configure_workflow(self):
+                jobA = Task(task_id="jobA", command="commandA --optionA=A")
+                return (jobA,)
+
+            def on_start(self):
+                jobB = Task(task_id="jobB", command="commandB --optionB=B")
+                self.get_task("jobA").add_next_task(jobB)
+                super().on_start()
+
+        with script_args(["--root-jobs"]):
+            manager = _TestManager()
+        manager.schedule_script = Mock()
+        manager.schedule_script.side_effect = ["999/1/1"]
+        manager._WorkFlowManager__on_start()
+
+        # first loop
+        self.assertTrue(manager.workflow_loop())
+        self.assertEqual(manager.schedule_script.call_count, 1)
+        manager.schedule_script.assert_any_call(
+            ["commandA", "--optionA=A"], tags=None, units=None, project_id=None,
+        )
+
+        # second loop
+        manager.schedule_script.side_effect = ["999/2/1"]
+        manager.is_finished = lambda x: "finished"
+        self.assertTrue(manager.workflow_loop())
+        self.assertEqual(manager.schedule_script.call_count, 2)
+
+        manager.schedule_script.assert_any_call(["commandB", "--optionB=B"], tags=None, units=None, project_id=None)
